package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.BeanUtils;
import org.springframework.jms.support.converter.MessageConversionException;

public class DefaultImplementationMapping {

	private final Map typeToImpl = new HashMap();

	private final Map implToType = new HashMap();
	
	public DefaultImplementationMapping()
	{
		addDefaultImplementation(Map.class, HashMap.class);
		addDefaultImplementation(ArrayList.class, List.class);
		addDefaultImplementation(HashSet.class, Set.class);
	}
	public void addDefaultImplementation(Class defaultImplementation,
			Class ofType) {
		typeToImpl.put(ofType, defaultImplementation);
		implToType.put(defaultImplementation, ofType);
	}
	
	public boolean containsImplMapping(Class clazz)
	{
		return typeToImpl.containsKey(clazz);
	}
	
	public Class getImplMapping(Class clazz)
	{
		return (Class) typeToImpl.get(clazz);
	}
	
	public Collection createCollection(Field targetField) {
		// TODO abstract out strategy to create specific collection

		if (containsImplMapping(targetField.getType())) {
			Class clazz = getImplMapping(targetField.getType());
			return (Collection) BeanUtils.instantiateClass(clazz);
		} else {
			throw new MessageConversionException(
					"Could not resovle target collection type for "
							+ targetField.getType());
		}
	}
	
	public Map createMap(Field targetField) {
		// TODO abstract out strategy to create specific collection

		if (containsImplMapping(targetField.getType())) {
			Class clazz = getImplMapping(targetField.getType());
			return (Map) BeanUtils.instantiateClass(clazz);
		} else {
			throw new MessageConversionException(
					"Could not resovle target collection type for "
							+ targetField.getType());
		}
	}
}
