package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Session;

import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;
import org.springframework.jms.support.JmsUtils;

public class ObjectBuilder implements Builder {

	public boolean canBuild(Class targetClass) {
		return true;
	}

	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String name, Class clazz,
			Object objectToConvert) {
		
		try {
			MapMessage messageToPopulate = null;

		if (name != null) {
			MapMessage nestedMapMessage = session.createMapMessage();
			Messages.setBooleanProperty(nestedMapMessage, "JMS_TIBCO_MSG_EXT",
					true);
			mm.setObject(name, nestedMapMessage);
			messageToPopulate = nestedMapMessage;
		} else {
			messageToPopulate = mm;
		}

		messageToPopulate.setString(conversionContext.getClassMapper()
				.getClassIdFieldName(), conversionContext.getClassMapper()
				.fromClass(objectToConvert.getClass()));

		Field[] fields = objectToConvert.getClass().getDeclaredFields();
		for (int i = 0; i < fields.length; i++) {
			Field field = fields[i];
			field.setAccessible(true);

			Object fieldVal = Fields.read(field, objectToConvert);
			String fieldName = field.getName();

			if (fieldVal != null) {
				conversionContext.convertToMessage(messageToPopulate, session,
                        fieldName,
                        fieldVal); 
			}
		}
		
		} catch  (JMSException e) { 
			JmsUtils.convertJmsAccessException(e);
		}

	}

	public void setValue(ConversionContext conversionContext,
			Object targetObject, Object val, Field targetField) {
		
		if (val instanceof MapMessage) {
			MapMessage mm = (MapMessage) val;
			targetField.setAccessible(true);
			try {
				Fields.write(targetField, targetObject, conversionContext
						.convertToObject(mm));
			} catch (JMSException e) {
				throw JmsUtils.convertJmsAccessException(e);
			}
		}

	}

}
