package org.spring.jmsinterop.builders;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import javax.jms.MapMessage;
import javax.jms.Session;

import org.spring.jmsinterop.converters.ConversionContext;
import org.spring.jmsinterop.util.Fields;
import org.spring.jmsinterop.util.Messages;

/**
 * Builder that works on primitive types and their wrapper classes, 
 * i.e. int, Integer; short, Short etc.
 *
 */
public class PrimitiveTypeBuilder implements Builder {

	private Map primitiveTypes = new HashMap();

	private Map primitiveWrapperTypes = new HashMap();

	public PrimitiveTypeBuilder() {
		InitMaps();
	}

	public boolean canBuild(Class targetClass) {
		String name = targetClass.getName();
		return primitiveTypes.containsKey(name)
				|| primitiveWrapperTypes.containsKey(name);
	}

	public void addValue(MapMessage mm, Session session,
			ConversionContext conversionContext, String fieldName, Class fieldClass,
			Object fieldVal) {

		Messages.setMessageObject(mm, fieldName, fieldVal);

	}

	public void setValue(ConversionContext conversionContext, Object targetObject,
			Object val, Field targetField) {

		Fields.write(targetField, targetObject, val);
	}

	private void InitMaps() {
		primitiveTypes.put("short", "");
		primitiveTypes.put("int", "");
		primitiveTypes.put("long", "");
		primitiveTypes.put("float", "");
		primitiveTypes.put("double", "");
		primitiveTypes.put("byte", "");
		primitiveTypes.put("boolean", "");
		primitiveTypes.put("char", "");

		primitiveWrapperTypes.put("java.lang.Short", "");
		primitiveWrapperTypes.put("java.lang.Integer", "");
		primitiveWrapperTypes.put("java.lang.Long", "");
		primitiveWrapperTypes.put("java.lang.Float", "");
		primitiveWrapperTypes.put("java.lang.Double", "");
		primitiveWrapperTypes.put("java.lang.Byte", "");
		primitiveWrapperTypes.put("java.lang.Boolean", "");
		primitiveWrapperTypes.put("java.lang.Character", "");

	}

}
