package org.spring.jmsinterop.converters;

import java.lang.reflect.Field;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import org.spring.jmsinterop.builders.Builder;
import org.spring.jmsinterop.builders.BuilderLookup;
import org.spring.jmsinterop.util.Fields;
import org.springframework.beans.BeanUtils;
import org.springframework.jms.support.converter.MessageConversionException;
import org.springframework.jms.support.converter.MessageConverter;
import org.springframework.util.ClassUtils;

public class ReflectionMessageConverter implements MessageConverter {

	
	private ConversionContext conversionContext;
	    
    private String converterId;

    private String converterIdFieldName;
    
	public ReflectionMessageConverter() {
        converterId = ClassUtils.getShortName(getClass());
        converterIdFieldName = "__ConverterId__";
	}

	// Properties 
	
	public ConversionContext getConversionContext() {
		return conversionContext;
	}


	public void setConversionContext(ConversionContext conversionContext) {
		this.conversionContext = conversionContext;
	}


	public String getConverterId() {
		return converterId;
	}


	public void setConverterId(String converterId) {
		this.converterId = converterId;
	}


	public String getConverterIdFieldName() {
		return converterIdFieldName;
	}

	
	// Methods
	
	public Message toMessage(Object objectToConvert, Session session) throws JMSException, MessageConversionException {
        if (objectToConvert == null)
        {
            throw new MessageConversionException("Can't convert null object");
        }

        try
        {
            return buildMessage(objectToConvert, session);
        }
        catch (Exception e)
        {
            throw new MessageConversionException("Could not convert object to message. Object = " + objectToConvert, e);
        }
	}
	
    protected Message buildMessage(Object objectToConvert, Session session) throws JMSException
    {
        
        Message message = conversionContext.convertToMessage(objectToConvert, session);
        message.setStringProperty(getConverterIdFieldName(), getConverterId());  
        return message;

        
    }


	public Object fromMessage(Message messageToConvert) throws JMSException, MessageConversionException {
        if (messageToConvert == null)
        {
            throw new MessageConversionException("Can't convert null message");
        }

        try
        {
                return buildObject(messageToConvert);
        }
        catch (Exception e)
        {
            throw new MessageConversionException("Could not convert message to object. Message =  " + messageToConvert, e);
        }
	}
	
	private Object buildObject(Message messageToConvert) throws JMSException {
		
		if (! (messageToConvert instanceof MapMessage))
		{
			throw new MessageConversionException(converterId + " requires a message of type MapMessage");            
		}
		MapMessage mm = (MapMessage)messageToConvert;
		
		return conversionContext.convertToObject(mm);
	}	

}
