package org.spring.jmsinterop.converters;

import java.util.Map;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;

import org.springframework.jms.support.converter.MessageConversionException;
import org.springframework.jms.support.converter.MessageConverter;
import org.springframework.jms.support.converter.SimpleMessageConverter;

public class SimpleMultiMessageConverter implements MessageConverter {

	private MessageConverter defaultMessageConverter = new SimpleMessageConverter();

	private ReflectionMessageConverter reflectionMessageConverter;

	public void setReflectionMessageConverter(
			ReflectionMessageConverter reflectionMessageConverter) {
		this.reflectionMessageConverter = reflectionMessageConverter;
	}

	public Object fromMessage(Message message) throws JMSException,
			MessageConversionException {
        if (message.getStringProperty(reflectionMessageConverter.getConverterIdFieldName()) != null)
        {
            return reflectionMessageConverter.fromMessage(message);
        }
        else
        {
            return defaultMessageConverter.fromMessage(message);
        }
	}

	public Message toMessage(Object objectToConvert, Session session)
			throws JMSException, MessageConversionException {
		if (objectToConvert.getClass().equals(String.class)
				|| Map.class.isAssignableFrom(objectToConvert.getClass())
				|| Byte[].class.isAssignableFrom(objectToConvert.getClass())) {
			return defaultMessageConverter.toMessage(objectToConvert, session);
		} else {
			return reflectionMessageConverter.toMessage(objectToConvert,
					session);
		}
	}

}
