package org.spring.jmsinterop.converters;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Session;

import org.spring.jmsinterop.bo.Portfolio;
import org.spring.jmsinterop.bo.Trade;
import org.spring.jmsinterop.bo.TradeRequest;
import org.spring.jmsinterop.bo.User;
import org.spring.jmsinterop.builders.BuilderLookup;
import org.springframework.jms.support.converter.MessageConverter;

import com.tibco.tibjms.TibjmsConnectionFactory;

import junit.framework.TestCase;

public class BaseMessageConverterTests extends TestCase {

	private boolean first = true;
    protected Session session;
    protected Connection connection;
    protected MessageConverter converter;
    
	public void setUp()
	{
		
		if (first)
		{
			ConnectionFactory connectionFactory = new TibjmsConnectionFactory("tcp://localhost:7222");
			try {
				connection = connectionFactory.createConnection();
				session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
				converter = getReflectionMessageConverter();
			} catch (JMSException e) {
				e.printStackTrace();
			}			
		}
		
		first = false;
	}
	
    protected TradeRequest createTradeRequest()
    {
        TradeRequest tradeRequest = new TradeRequest();
        tradeRequest.setAccountName("ACCT-123");
        tradeRequest.setBuyRequest(true);
        tradeRequest.setOrderType("MARKET");
        tradeRequest.setQuantity(314000000);
        tradeRequest.setRequestId("REQ-1");
        tradeRequest.setTicker("CSCO");
        tradeRequest.setUserName("Joe Trader");
        return tradeRequest;
    }
    
    protected Portfolio createPortfolio()
    {
        Portfolio portfolio = new Portfolio();
        User user = new User();
        user.setUserName("Joe Trader");
        user.setAccountName("ACCT-123");
        portfolio.setUser(user);
        portfolio.setPortfolioName("High Risk Trading Account");

        Trade trade = createTrade();
        portfolio.getTrades().add(trade);
        Trade anotherTrade = createTrade();
        anotherTrade.setTicker("SUNW");
        portfolio.getTrades().add(anotherTrade);
        return portfolio;
    }

    private Trade createTrade()
    {
        Trade trade = new Trade();
        trade.setOrderType("Market");
        trade.setPrice(28.3);
        trade.setQuantity(1000);
        trade.setTicker("CSCO");
        return trade;
    }

	protected MessageConverter getReflectionMessageConverter() {
	    ReflectionMessageConverter converter = new ReflectionMessageConverter();
	    SimpleClassMapper classMapper = new SimpleClassMapper();
	    classMapper.setDefaultPackage("org.spring.jmsinterop.bo");
	    ConversionContext conversionContext = new ConversionContext();
	    conversionContext.setClassMapper(classMapper);
	    BuilderLookup builderLookup = new BuilderLookup();
	    conversionContext.setBuilderLookup(builderLookup);
	    converter.setConversionContext(conversionContext);
	    return converter;
	}
}
