package org.spring.jmsinterop.converters;

import java.util.List;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;

import org.spring.jmsinterop.bo.Person;
import org.spring.jmsinterop.bo.Portfolio;
import org.spring.jmsinterop.bo.TradeRequest;
import org.spring.jmsinterop.bo.User;
import org.springframework.jms.support.converter.MessageConversionException;
import org.springframework.jms.support.converter.MessageConverter;

public class ReflectionMessageConverterTests extends BaseMessageConverterTests {

	public void testUser() throws MessageConversionException, JMSException
	{
		User user = new User();
        user.setAccountName("ACCT-123");
        user.setUserName("Joe Trader");
        
        converter = getReflectionMessageConverter();

        Message message = converter.toMessage(user, session);
        this.assertTrue(message instanceof MapMessage);
        MapMessage mm = (MapMessage)message;
        this.assertEquals("ACCT-123", mm.getString("accountName"));
        Object userObject = converter.fromMessage(mm);
        this.assertTrue(userObject instanceof User);
        user = (User)userObject;
        this.assertEquals("ACCT-123", user.getAccountName());
        
        
	}
	
	public void testTradeRequest() throws MessageConversionException, JMSException
	{
		converter = getReflectionMessageConverter();
		TradeRequest tradeRequest = createTradeRequest();

		Message message = converter.toMessage(tradeRequest, session);
		assertTrue(message instanceof MapMessage);
		MapMessage mm = (MapMessage)message;
		assertEquals("ACCT-123", mm.getString("accountName"));
		Object tradeRequestObject = converter.fromMessage(mm);
		assertTrue(tradeRequestObject instanceof TradeRequest);
		tradeRequest = (TradeRequest)tradeRequestObject;
		assertEquals("ACCT-123", tradeRequest.getAccountName());
		
	}
	
	public void testPortfolio() throws MessageConversionException, JMSException
	{
		converter = getReflectionMessageConverter();
		Portfolio portfolio = createPortfolio();
        Message message = converter.toMessage(portfolio, session);
		assertTrue(message instanceof MapMessage);
		MapMessage mm = (MapMessage)message;
        assertEquals("High Risk Trading Account", mm.getString("portfolioName"));
        Object userMessageObject = mm.getObject("user");
        assertTrue(userMessageObject instanceof MapMessage);
                        
        MapMessage userMessage = (MapMessage)userMessageObject;
        assertEquals("ACCT-123", userMessage.getString("accountName"));
       
        Object portfolioObject = converter.fromMessage(mm);
        assertTrue(portfolioObject instanceof Portfolio);
        portfolio = (Portfolio)portfolioObject;
        assertEquals("High Risk Trading Account", portfolio.getPortfolioName());
        assertEquals("ACCT-123", portfolio.getUser().getAccountName());
        List tradeList = portfolio.getTrades();
        assertEquals(2, tradeList.size());
	}

	public void testPerson() throws MessageConversionException, JMSException
	{
		Person p1 = new Person();
		p1.setFirstName("Joe");
		Person p2 = new Person();
		p2.setFirstName("Mary");
		p1.setFriend(p2);
        Message message = converter.toMessage(p1, session);
		assertTrue(message instanceof MapMessage);
		MapMessage mm = (MapMessage)message;
        assertEquals("Joe", mm.getString("firstName"));

        Object personObject = converter.fromMessage(mm);
        assertTrue(personObject instanceof Portfolio);
        p1 = (Person)personObject;
        assertEquals("Joe", p1.getFirstName());
        assertEquals("Mary", p1.getFriend().getFirstName());
   
		
		
	}
	public void testPersonWithCircularReference()
	{
		Person p1 = new Person();
		Person p2 = new Person();
		p1.setFriend(p2);
		p2.setFriend(p1);
		converter = getReflectionMessageConverter();
		try {
			Message message = converter.toMessage(p1, session);
			fail("Expected Circular Reference");
		} catch (MessageConversionException e) {
			//ok
		} catch (JMSException e) {
			fail("Did not expect JMSException");
		}
	}
}
