using System;
using System.Globalization;
using System.Reflection;
using Spring.JmsInterop.Converters;
using TIBCO.EMS;

namespace Spring.JmsInterop.Builders
{
    public class DecimalBuilder : IBuilder
    {
        private CultureInfo cultureInfo;

        private static readonly int DEFAULT_NUMBER_DECIMAL_DIGITS = 8;

        public bool CanBuild(Type targetType)
        {
            return targetType.Equals(typeof (Decimal));
        }

        public void AddValue(MapMessage mm, Session session, ConversionContext conversionContext, string fieldName,
                             Type fieldType, object fieldVal)
        {
            NumberFormatInfo nfi = DecimalFormatter.NumberFormat;
            decimal realVal = (decimal) fieldVal;
            string sVal = realVal.ToString("N", nfi);
            mm.SetString(fieldName, sVal);
        }

        public void SetValue(MapMessage mapMessage, ConversionContext conversionContext, object targetObject,
                             object fieldVal, FieldInfo targetFieldInfo)
        {
            NumberFormatInfo nfi = DecimalFormatter.NumberFormat;
            decimal dVal = Decimal.Parse(fieldVal.ToString(), nfi);
            targetFieldInfo.SetValue(targetObject, dVal);
        }

        protected virtual CultureInfo DecimalFormatter
        {
            get
            {
                if (cultureInfo == null)
                {
                    cultureInfo = CultureInfo.CurrentCulture.Clone() as CultureInfo;
                    cultureInfo.NumberFormat.NumberDecimalDigits = DEFAULT_NUMBER_DECIMAL_DIGITS;
                    cultureInfo.NumberFormat.NumberGroupSeparator = "";
                }
                return cultureInfo;
            }
            set { cultureInfo = value; }
        }
    }
}