using System;
using System.Collections;
using System.Reflection;
using Spring.JmsInterop.Converters;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Builders
{
    public class DictionaryBuilder : IBuilder
    {
        public bool CanBuild(Type targetType)
        {
            return (typeof (IDictionary).IsAssignableFrom(targetType));
        }

        public void AddValue(MapMessage mm, Session session, ConversionContext conversionContext, 
                             string fieldName, Type fieldType, object fieldVal)
        {
            IDictionary dict = fieldVal as IDictionary;
            MapMessage dictMsg = session.CreateMapMessage();
            dictMsg.SetBooleanProperty("JMS_TIBCO_MSG_EXT", true);
            dictMsg.SetString(conversionContext.TypeMapper.TypeIdFieldName,
                              conversionContext.TypeMapper.FromType(dict.GetType()));

            mm.SetObject(fieldName, dictMsg);
            foreach (DictionaryEntry entry in dict)
            {
                string key = entry.Key as string;
                if (key == null)
                {
                    throw new MessageConversionException("Cannot convert non-String key of type [" +
                                                         (entry.Key != null ? entry.Key.GetType().FullName : null) +
                                                         "] to MapMessage entry");
                }

                conversionContext.ConvertToMessage(dictMsg, session,
                                                   key,
                                                   entry.Value);


                //mm.SetObject(key, conversionContext.ToMessage(entry.Value, session));
            }
        }

        public void SetValue(MapMessage mapMessage, ConversionContext conversionContext, object targetObject,
                             object fieldVal, FieldInfo targetFieldInfo)
        {
            MapMessage mm = fieldVal as MapMessage;
            if (mm != null)
            {
                targetFieldInfo.SetValue(targetObject, BuildHashtable(conversionContext, mm));
            }
        }

        private object BuildHashtable(ConversionContext conversionContext, MapMessage mm)
        {
            IDictionary dictionary = new Hashtable();
            IEnumerator e = mm.MapNames;
            while (e.MoveNext())
            {
                String key = (String) e.Current;
                //TODO refactor to common location of definition.
                if (key != "__TypeId__")
                {
                    dictionary.Add(key, mm.GetObject(key));
                }
            }
            return dictionary;
        }
    }
}