using System;
using System.Collections;
using System.Reflection;
using Spring.JmsInterop.Converters;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Builders
{
    public class ListBuilder : AbstractCollectionBuilder
    {
        
        public override bool CanBuild(Type targetType)
        {
            return (typeof(IList).IsAssignableFrom(targetType));
        }

        public override void SetValue(MapMessage mapMessage, ConversionContext conversionContext, object targetObject, object fieldVal, FieldInfo targetFieldInfo)
        {
            MapMessage mm = fieldVal as MapMessage;
            if (mm != null)
            {
                targetFieldInfo.SetValue(targetObject, BuildCollection(conversionContext, mm));
            }
        }

        private object BuildCollection(ConversionContext conversionContext, MapMessage mm)
        {

            IList targetList = new ArrayList();

            for (int i = 0; i < mm.FieldCount; i++)
            {

                string jmsFieldName = i.ToString();
                object val = null;

                if (mm.ItemExists(jmsFieldName))
                {
                    val = mm.GetObject(jmsFieldName);
                }
                if (val != null)
                {
                    MapMessage nestedMapMessage = val as MapMessage;
                    if (nestedMapMessage != null)
                    {
                        object targetObject = conversionContext.ConvertToObject(nestedMapMessage);
                        targetList.Add(targetObject);
                    }
                    else
                    {
                        throw new MessageConversionException("Conversion of simple type in lists not supported yet.");
                    }
                }
            }
            return targetList;
        }
    }
}
