using System;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    public class ReflectionMessageConverter : IMessageConverter
    {
        private ConversionContext conversionContext;
        
        private string converterId;

        private string converterIdFieldName;
        

        public ReflectionMessageConverter()
        {
            converterId = GetType().Name;
            converterIdFieldName = "__ConverterId__";
        }

        #region Properties

        public ConversionContext ConversionContext
        {
            get { return conversionContext; }
            set { conversionContext = value; }
        }

        public string ConverterId
        {
            get { return converterId; }
            set { converterId = value; }
        }


        public string ConverterIdFieldName
        {
            get { return converterIdFieldName; }
        }

        #endregion

        public Message ToMessage(object objectToConvert, Session session)
        {
            if (objectToConvert == null)
            {
                throw new MessageConversionException("Can't convert null object");
            }

            try
            {
                return BuildMessage(objectToConvert, session);
            }
            catch (Exception e)
            {
                throw new MessageConversionException("Could not convert object to message. Object = " + objectToConvert, e);
            }

        }

        protected virtual Message BuildMessage(object objectToConvert, Session session)
        {

            Message message = conversionContext.ConvertToMessage(objectToConvert, session);
            message.SetStringProperty(ConverterIdFieldName, ConverterId);
            return message;
        }


        public object FromMessage(Message messageToConvert)
        {
            if (messageToConvert == null)
            {
                throw new MessageConversionException("Can't convert null message");
            }

            try
            {
                    return BuildObject(messageToConvert);
            }
            catch (Exception e)
            {
                throw new MessageConversionException("Could not convert message to object. Message =  " + messageToConvert, e);
            }

            
            
        }

        private object BuildObject(Message messageToConvert)
        {
            MapMessage mm = messageToConvert as MapMessage;
            if (mm == null)
            {
                throw new MessageConversionException(converterId + " requires a message of type MapMessage");
            }
            return conversionContext.ConvertToObject(mm);
        }



    }
}
