using System;
using System.Collections;
using System.Windows.Forms;
using log4net;
using Spring.Context.Support;
using Spring.JmsInterop.Bo;

namespace Spring.JmsInterop.UI
{
    public partial class StockForm : Form
    {
        #region Logging Definition

        private static readonly ILog log = LogManager.GetLogger(typeof (StockForm));

        #endregion

        private StockController stockController;

        public StockForm()
        {
            InitializeComponent();
            //Note, support for transparant winform dependency injection under development.
            stockController = ContextRegistry.GetContext()["stockController"] as StockController;
            stockController.StockForm = this;
        }

        public StockController Controller
        {
            set { stockController = value; }
        }

        private void OnSendTradeRequest(object sender, EventArgs e)
        {
            //In this simple example no data is collected from the view.
            //Instead a hardcoded trade request is created in the controller.
            tradeRequestStatusTextBox.Text = "Request Pending...";
            stockController.SendTradeRequest();
            log.Info("Sent trade request.");
        }

        private void OnGetPortfolio(object sender, EventArgs e)
        {
            this.portfolioListBox.Items.Clear();
            this.portfolioListBox.Items.Add("Retrieving...");
            stockController.SendPortfolioRequest();
        }

        public void UpdateTrade(Trade trade)
        {
            Invoke(new MethodInvoker(
                       delegate
                           {
                               tradeRequestStatusTextBox.Text = "Confirmed. " + trade.Ticker + " " + trade.Price;
                           }));
        }

        public void UpdateMarketData(IDictionary marketDataDict)
        {
            Invoke(new MethodInvoker(
                       delegate
                           {
                               
                               this.marketDataListBox.Items.Add(marketDataDict["TICKER"] + " " + marketDataDict["PRICE"]);
                           }));
        }

        public void UpdatePortfolio(Portfolio portfolio)
        {
            Invoke(new MethodInvoker(
                       delegate
                           {
                               this.portfolioListBox.Items.Clear();
                               this.portfolioListBox.Items.Add(portfolio.PortfolioName);
                               foreach (Trade trade in portfolio.Trades)
                               {

                                   this.portfolioListBox.Items.Add(
                                       string.Format("TICKER = {0}, PRICE = {1}, QUANTITY = {2}",
                                                     trade.Ticker, trade.Price, trade.Quantity));
                               }                              
                           }));
        }

    }
}