using NUnit.Framework;
using Spring.JmsInterop.Bo;
using Spring.JmsInterop.Builders;
using Spring.Messaging.Tibco.Ems.Support.Converter;
using TIBCO.EMS;

namespace Spring.JmsInterop.Converters
{
    [TestFixture]
    public class BaseMessageConverterTests
    {
        protected Session session;
        protected Connection connection;

        [SetUp]
        public void SetUp()
        {
            ConnectionFactory factory = new ConnectionFactory("tcp://localhost:7222");

            connection = factory.CreateConnection();

            session = connection.CreateSession(false, Session.AUTO_ACKNOWLEDGE);
        }

        [TearDown]
        public void TearDown()
        {
            connection.Close();
        }

        protected TradeRequest CreateTradeRequest()
        {
            TradeRequest tradeRequest = new TradeRequest();
            tradeRequest.AccountName = "ACCT-123";
            tradeRequest.BuyRequest = true;
            tradeRequest.OrderType = "MARKET";
            tradeRequest.Quantity = 314000000;
            tradeRequest.RequestId = "REQ-1";
            tradeRequest.Ticker = "CSCO";
            tradeRequest.UserName = "Joe Trader";
            return tradeRequest;
        }

        protected void TestTradeRequestConversion(IMessageConverter converter)
        {
            TradeRequest tradeRequest = CreateTradeRequest();
            Message message = converter.ToMessage(tradeRequest, session);
            MapMessage mm = message as MapMessage;
            Assert.IsNotNull(mm);
            Assert.AreEqual("ACCT-123", mm.GetString("accountName"));
            tradeRequest = converter.FromMessage(mm) as TradeRequest;
            Assert.IsNotNull(tradeRequest);
            Assert.AreEqual("ACCT-123", tradeRequest.AccountName);
        }

        protected ReflectionMessageConverter GetReflectionMessageConverter()
        {
            ReflectionMessageConverter converter = new ReflectionMessageConverter();
            SimpleTypeMapper typeMapper = new SimpleTypeMapper();
            typeMapper.DefaultNamespace = "Spring.JmsInterop.Bo";
            typeMapper.DefaultAssemblyName = "Spring.JmsInterop";
            /*
            converter.TypeMapper = typeMapper;
            BuilderLookup builderLookup = new BuilderLookup();
            converter.BuilderLookup = builderLookup;
             */
            ConversionContext conversionContext = new ConversionContext();
            conversionContext.TypeMapper = typeMapper;
            converter.ConversionContext = conversionContext;

            return converter;
        }




        protected void TestPorfolioConversion(IMessageConverter converter, Portfolio portfolio)
        {
            Message message = converter.ToMessage(portfolio, session);
            MapMessage mm = message as MapMessage;
            Assert.IsNotNull(mm);
            Assert.AreEqual("High Risk Trading Account", mm.GetString("portfolioName"));
            MapMessage userMessage = mm.GetObject("user") as MapMessage;
            Assert.IsNotNull(userMessage);
            Assert.AreEqual("ACCT-123", userMessage.GetString("accountName"));
           
            portfolio = converter.FromMessage(mm) as Portfolio;
            Assert.IsNotNull(portfolio);
            Assert.AreEqual("High Risk Trading Account", portfolio.PortfolioName);
            Assert.AreEqual("ACCT-123", portfolio.User.AccountName);
        }

        protected Portfolio CreatePortfolio()
        {
            Portfolio portfolio = new Portfolio();
            User user = new User();
            user.UserName = "Joe Trader";
            user.AccountName = "ACCT-123";
            portfolio.User = user;
            portfolio.PortfolioName = "High Risk Trading Account";

            Trade trade = CreateTrade();
            portfolio.Trades.Add(trade);
            
            return portfolio;
        }

        private Trade CreateTrade()
        {
            Trade trade = new Trade();
            trade.OrderType = "Market";
            trade.Price = 28.3;
            trade.Quantity = 1000;
            trade.Ticker = "CSCO";
            return trade;
        }
    }
}
